@@ -1,4 +1,5 @@
-from ciscoconfparse import *
+from __future__  import absolute_import
+from ciscoconfparse.ciscoconfparse import *
 
 """ __init__.py - Parse, Query, Build, and Modify IOS-style configurations
      Copyright (C) 2007-2015 David Michael Pennington
