@@ -1,13 +1,14 @@
+from __future__  import absolute_import
 from operator import methodcaller, attrgetter
 from abc import ABCMeta, abstractmethod
 from copy import deepcopy
 import re
 import os
 
-from ccp_util import IPv4Obj
+from ciscoconfparse.ccp_util import IPv4Obj
 
-""" ccp_abc.py - Parse, Query, Build, and Modify IOS-style configurations
-     Copyright (C) 2014-2015 David Michael Pennington
+r""" ccp_abc.py - Parse, Query, Build, and Modify IOS-style configurations
+     Copyright (C) 2014-2015, 2019 David Michael Pennington
 
      This program is free software: you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
@@ -197,7 +198,7 @@ class BaseCfgLine(object):
         """unconftext is defined during special method calls.  Do not assume it
         is automatically populated."""
         ## remove any preceeding "no "
-        conftext = re.sub("\s*no\s+", "", unconftext)
+        conftext = re.sub(r"\s*no\s+", "", unconftext)
         myindent = self.parent.child_indent
         self.uncfgtext = myindent * " " + "no " + conftext
 
@@ -250,7 +251,7 @@ class BaseCfgLine(object):
            ...     obj.delete_children_matching(r'description')
            >>>
            >>> for line in parse.ioscfg:
-           ...     print line
+           ...     print(line)
            ...
            !
            interface Serial1/0
@@ -273,15 +274,15 @@ class BaseCfgLine(object):
     def insert_before(self, insertstr):
         """insert_before()"""
         ## BaseCfgLine.insert_before(), insert a single line before this object
-        local_atomic=False
-        retval = self.confobj.insert_before(self, insertstr, atomic=local_atomic)
+        retval = self.confobj.insert_before(self, insertstr, 
+            atomic=False)
         return retval
 
     def insert_after(self, insertstr):
         """insert_after()"""
         ## BaseCfgLine.insert_after(), insert a single line after this object
-        local_atomic=False
-        retval = self.confobj.insert_after(self, insertstr, atomic=local_atomic)
+        retval = self.confobj.insert_after(self, insertstr, 
+            atomic=False)
         return retval
 
     def append_to_family(self, insertstr, indent=-1, auto_indent_width=1, 
@@ -320,7 +321,7 @@ class BaseCfgLine(object):
            ...     obj.append_to_family(' carrier-delay msec 500')
            >>>
            >>> for line in parse.ioscfg:
-           ...     print line
+           ...     print(line)
            ...
            !
            interface Serial1/0
@@ -341,15 +342,14 @@ class BaseCfgLine(object):
 
         ## BaseCfgLine.append_to_family(), insert a single line after this 
         ##  object's children
-        local_atomic=False
         try:
-            last_child = self.children[-1]
+            last_child = self.all_children[-1]
             retval = self.confobj.insert_after(last_child, insertstr, 
-                atomic=local_atomic)
+                atomic=False)
         except IndexError:
             # The object has no children
             retval = self.confobj.insert_after(self, insertstr, 
-                atomic=local_atomic)
+                atomic=False)
 
         return retval
 
@@ -388,9 +388,9 @@ class BaseCfgLine(object):
            >>> parse = CiscoConfParse(config)
            >>>
            >>> for obj in parse.find_objects('Serial'):
-           ...     print "OLD", obj.text
+           ...     print("OLD {}".format(obj.text))
            ...     obj.replace(r'Serial1', r'Serial0')
-           ...     print "  NEW", obj.text
+           ...     print("  NEW {}".format(obj.text))
            OLD interface Serial1/0
              NEW interface Serial0/0
            OLD interface Serial1/1
@@ -433,9 +433,9 @@ class BaseCfgLine(object):
            >>> parse = CiscoConfParse(config)
            >>>
            >>> for obj in parse.find_objects('Serial'):
-           ...     print "OLD", obj.text
-           ...     obj.re_sub(r'Serial1', r'Serial0')
-           ...     print "  NEW", obj.text
+           ...     print("OLD {}".format(obj.text))
+           ...     obj.replace(r'Serial1', r'Serial0')
+           ...     print("  NEW {}".format(obj.text))
            OLD interface Serial1/0
              NEW interface Serial0/0
            OLD interface Serial1/1
@@ -458,7 +458,7 @@ class BaseCfgLine(object):
         return retval
 
     def re_match(self, regex, group=1, default=""):
-        """Use ``regex`` to search the :class:`~models_cisco.IOSCfgLine` text and return the regular expression group, at the integer index.
+        r"""Use ``regex`` to search the :class:`~models_cisco.IOSCfgLine` text and return the regular expression group, at the integer index.
 
         Args:
             - regex (str): A string or python regular expression, which should be matched.  This regular expression should contain parenthesis, which bound a match group.
@@ -490,7 +490,7 @@ class BaseCfgLine(object):
            >>> for obj in parse.find_objects(r'ip\saddress'):
            ...     netmask = obj.re_match(r'1\.1\.1\.5\s(\S+)')
            >>>
-           >>> print "The netmask is", netmask
+           >>> print("The netmask is", netmask)
            The netmask is 255.255.255.252
            >>>
         """
@@ -518,21 +518,26 @@ class BaseCfgLine(object):
             return self.text
         return default
 
-    def re_search_children(self, regex):
+    def re_search_children(self, regex, recurse=False):
         """Use ``regex`` to search the text contained in the children of 
         this :class:`~models_cisco.IOSCfgLine`.
 
         Args:
             - regex (str): A string or python regular expression, which should be matched.  
+            - recurse (bool): Set True if you want to search all children (children, grand children, great grand children, etc...)
 
         Returns:
             - list.  A list of matching :class:`~models_cisco.IOSCfgLine` objects which matched.  If there is no match, an empty :py:func:`list` is returned.
 
         """
-        return [cobj for cobj in self.children if cobj.re_search(regex)]
+        if recurse is False:
+            return [cobj for cobj in self.children if cobj.re_search(regex)]
+        else:
+            return [cobj for cobj in self.all_children if cobj.re_search(regex)]
 
-    def re_match_typed(self, regex, group=1, result_type=str, default=''):
-        """Use ``regex`` to search the :class:`~models_cisco.IOSCfgLine` text 
+    def re_match_typed(self, regex, group=1, untyped_default=False, 
+        result_type=str, default=''):
+        r"""Use ``regex`` to search the :class:`~models_cisco.IOSCfgLine` text 
         and return the contents of the regular expression group, at the 
         integer ``group`` index, cast as ``result_type``; if there is no match, 
         ``default`` is returned.
@@ -543,9 +548,10 @@ class BaseCfgLine(object):
             - group (int): An integer which specifies the desired regex group to be returned.  ``group`` defaults to 1.
             - result_type (type): A type (typically one of: ``str``, ``int``, ``float``, or ``IPv4Obj``).  All returned values are cast as ``result_type``, which defaults to ``str``.
             - default (any): The default value to be returned, if there is no match.
+            - untyped_default (bool): Set True if you don't want the default value to be typed
 
         Returns:
-            - ``result_type``.  The text matched by the regular expression group; if there is no match, ``default`` is returned.  All values are cast as ``result_type``.
+            - ``result_type``.  The text matched by the regular expression group; if there is no match, ``default`` is returned.  All values are cast as ``result_type``, unless `untyped_default` is True.
 
         This example illustrates how you can use 
         :func:`~models_cisco.IOSCfgLine.re_match_typed` to build an 
@@ -574,7 +580,7 @@ class BaseCfgLine(object):
            ...     slot = obj.re_match_typed(regex=r'Serial(\d+)',
            ...         result_type=int,
            ...         default=-1)
-           ...     print "Interface {0} is in slot {1}".format(name, slot)
+           ...     print("Interface {0} is in slot {1}".format(name, slot))
            ...
            Interface Serial1/0 is in slot 1
            Interface Serial2/0 is in slot 2
@@ -585,13 +591,15 @@ class BaseCfgLine(object):
         if not (mm is None):
             if not (mm.group(group) is None):
                 return result_type(mm.group(group))
-            else:
-                return result_type(default)
-        return result_type(default)
+
+        if untyped_default:
+            return default
+        else:
+            return result_type(default)
 
     def re_match_iter_typed(self, regex, group=1, result_type=str, default='', 
-        all_children=False):
-        """Use ``regex`` to search the children of 
+        untyped_default=False, recurse=False):
+        r"""Use ``regex`` to search the children of 
         :class:`~models_cisco.IOSCfgLine` text and return the contents of 
         the regular expression group, at the integer ``group`` index, cast as 
         ``result_type``; if there is no match, ``default`` is returned.
@@ -602,10 +610,12 @@ class BaseCfgLine(object):
             - group (int): An integer which specifies the desired regex group to be returned.  ``group`` defaults to 1.
             - result_type (type): A type (typically one of: ``str``, ``int``, ``float``, or :class:`~ccp_util.IPv4Obj`).         All returned values are cast as ``result_type``, which defaults to ``str``.
             - default (any): The default value to be returned, if there is no match.
-            - all_children (bool): Set True if you want to search all children (children, grand children, great grand children, etc...)
+            - recurse (bool): Set True if you want to search all children (children, grand children, great grand children, etc...)
+            - untyped_default (bool): Set True if you don't want the default value to be typed
 
         Returns:
-            - ``result_type``.  The text matched by the regular expression group; if there is no match, ``default`` is returned.  All values are cast as ``result_type``.
+            - ``result_type``.  The text matched by the regular expression group; if there is no match, ``default`` is returned.  All values are cast as ``result_type``, unless `untyped_default` is True.
+
         - NOTE: This loops through the children (in order) and returns when the regex hits its first match.
 
         This example illustrates how you can use 
@@ -627,7 +637,7 @@ class BaseCfgLine(object):
            >>> INTF_RE = re.compile(r'interface\s\S+')
            >>> ADDR_RE = re.compile(r'ip\saddress\s(\S+\s+\S+)')
            >>> for obj in parse.find_objects(INTF_RE):
-           ...     print obj.text, obj.re_match_iter_typed(ADDR_RE, result_type=IPv4Obj)
+           ...     print("{} {}".format(obj.text, obj.re_match_iter_typed(ADDR_RE, result_type=IPv4Obj)))
            interface Serial1/0 <IPv4Obj 1.1.1.1/30>
            interface Serial2/0 <IPv4Obj 1.1.1.5/30>
            >>>
@@ -635,26 +645,34 @@ class BaseCfgLine(object):
         ## iterate through children, and return the matching value 
         ##  (cast as result_type) from the first child.text that matches regex
 
-        if (default is True):
+        #if (default is True):
             ## Not using self.re_match_iter_typed(default=True), because I want
             ##   to be sure I build the correct API for match=False
             ##
             ## Ref IOSIntfLine.has_dtp for an example of how to code around
             ##   this while I build the API
-            raise NotImplementedError
+        #    raise NotImplementedError
 
-        if all_children is False:
+        if recurse is False:
             for cobj in self.children:
                 mm = re.search(regex, cobj.text)
                 if not (mm is None):
                     return result_type(mm.group(group))
-            return result_type(default)
+            ## Ref Github issue #121
+            if untyped_default:
+                return default
+            else:
+                return result_type(default)
         else:
             for cobj in self.all_children:
                 mm = re.search(regex, cobj.text)
                 if not (mm is None):
                     return result_type(mm.group(group))
-            return result_type(default)
+            ## Ref Github issue #121
+            if untyped_default:
+                return default
+            else:
+                return result_type(default)
 
     def reset(self):
         # For subclass APIs
