@@ -1,21 +1,28 @@
-from collections import MutableSequence
+from __future__  import absolute_import
 import itertools
 import socket
+import time
 import sys
 import re
 import os
 
-from protocol_values import ASA_TCP_PORTS, ASA_UDP_PORTS
+if (sys.version_info>=(3, 0, 0,)):
+    from collections.abc import MutableSequence
+else:
+    ## This syntax is not supported in Python 3...
+    from collections import MutableSequence
+
+from ciscoconfparse.protocol_values import ASA_TCP_PORTS, ASA_UDP_PORTS
 from dns.exception import DNSException
 from dns.resolver import Resolver
-from dns import reversename, query
+from dns import reversename, query, zone
 
 if sys.version_info[0] < 3:
     from ipaddr import IPv4Network, IPv6Network, IPv4Address, IPv6Address
 else:
     from ipaddress import IPv4Network, IPv6Network, IPv4Address, IPv6Address
 """ ccp_util.py - Parse, Query, Build, and Modify IOS-style configurations
-     Copyright (C) 2014-2015 David Michael Pennington
+     Copyright (C) 2014-2015, 2018-2019 David Michael Pennington
 
      This program is free software: you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
@@ -77,14 +84,14 @@ _RGX_CISCO_RANGE = re.compile(_CISCO_RANGE_STR)
 def is_valid_ipv4_addr(input=""):
     """Check if this is a valid IPv4 string"""
     assert input!=""
-    if _RGX_IPV4ADDR(input):
+    if _RGX_IPV4ADDR.search(input):
         return True
     return False
 
 def is_valid_ipv6_addr(input=""):
     """Check if this is a valid IPv6 string"""
     assert input!=""
-    if _RGX_IPV6ADDR(input):
+    if _RGX_IPV6ADDR.search(input):
         return True
     return False
 
@@ -104,7 +111,8 @@ class IPv4Obj(object):
         - as_binary_tuple (tuple): The address as a tuple of zero-padded binary strings
         - as_hex_tuple (tuple): The address as a tuple of zero-padded 8-bit hex strings
         - as_decimal (int): The ip address as a decimal integer
-        - network (str): A string representing the network address
+        - network : An IPv4Network object
+        - network_object : An IPv4Network object
         - netmask (str): A string representing the netmask
         - prefixlen (int): An integer representing the length of the netmask
         - prefixlength (int): An integer representing the length of the netmask
@@ -115,6 +123,27 @@ class IPv4Obj(object):
     Returns:
         - an instance of :class:`~ccp_util.IPv4Obj`.
 
+>>> from ciscoconfparse.ccp_util import IPv4Obj
+>>> net = IPv4Obj('172.16.1.0/24')
+>>> net
+<IPv4Obj 172.16.1.0/24>
+>>> net.ip
+IPv4Address('172.16.1.0')
+>>> net.ip + 1
+IPv4Address('172.16.1.1')
+>>> str(net.ip+1)
+'172.16.1.1'
+>>> net.network
+IPv4Address('172.16.1.0')
+>>> net.network_object
+IPv4Network('172.16.1.0/24')
+>>> str(net.network_object)
+'172.16.1.0/24'
+>>> net.prefixlen
+24
+>>> net.network_object.iterhosts()
+<generator object iterhosts at 0x7f00bfcce730>
+>>>
     """
 
     def __init__(self, arg='127.0.0.1/32', strict=False):
@@ -140,6 +169,10 @@ class IPv4Obj(object):
                 self.network_object = IPv4Network(str(arg)+'/32')
                 self.ip_object = IPv4Address(str(arg).split('/')[0])
                 return None
+            elif isinstance(arg, int):
+                self.ip_object = IPv4Address(arg)
+                self.network_object = IPv4Network(str(self.ip_object)+'/32', strict=False)
+                return None
             else:
                 raise ValueError(
                     "IPv4Obj doesn't understand how to parse {0}".format(arg))
@@ -182,19 +215,19 @@ class IPv4Obj(object):
 
     def __gt__(self, val):
         try:
-            val_prefixlen = int(getattr(val, 'prefixlen'))
+            val_in_self = self.__contains__(val)
             val_nobj = getattr(val, 'network_object')
+            self_nobj = getattr(self, 'network_object')
+            val_dec = getattr(val, 'as_decimal')
+            self_dec = getattr(self, 'as_decimal')
 
-            self_nobj = self.network_object
-            if (self.network_object.prefixlen < val_prefixlen):
-                # Sort shorter masks as higher...
-                return True
-            elif (self.network_object.prefixlen > val_prefixlen):
+            if val_in_self:
+                # Sort shorter masks as lower...
                 return False
-            elif (self_nobj > val_nobj):
-                # If masks are equal, rely on Google's sorting...
-                return True
-            return False
+            elif self_nobj==val_nobj:
+                return (self_dec > val_dec)
+            else:
+                return (self_nobj > val_nobj)
         except:
             errmsg = "{0} cannot compare itself to '{1}'".format(
                 self.__repr__(), val)
@@ -202,24 +235,34 @@ class IPv4Obj(object):
 
     def __lt__(self, val):
         try:
-            val_prefixlen = int(getattr(val, 'prefixlen'))
+            val_in_self = self.__contains__(val)
             val_nobj = getattr(val, 'network_object')
+            self_nobj = getattr(self, 'network_object')
+            val_dec = getattr(val, 'as_decimal')
+            self_dec = getattr(self, 'as_decimal')
 
-            self_nobj = self.network_object
-            if (self.network_object.prefixlen > val_prefixlen):
+            if val_in_self:
                 # Sort shorter masks as lower...
                 return True
-            elif (self.network_object.prefixlen < val_prefixlen):
-                return False
-            elif (self_nobj < val_nobj):
-                # If masks are equal, rely on Google's sorting...
-                return True
-            return False
+            elif self_nobj==val_nobj:
+                return (self_dec < val_dec)
+            else:
+                return (self_nobj < val_nobj)
         except:
             errmsg = "{0} cannot compare itself to '{1}'".format(
                 self.__repr__(), val)
             raise ValueError(errmsg)
 
+    def __add__(self, val):
+        """Add an integer to IPv4Obj() and return an integer"""
+        assert isinstance(val, int)
+        return self.as_decimal + val
+
+    def __sub__(self, val):
+        """Subtract an integer from IPv4Obj() and return an integer"""
+        assert isinstance(val, int)
+        return self.as_decimal - val
+
     def __contains__(self, val):
         # Used for "foo in bar"... python calls bar.__contains__(foo)
         try:
@@ -274,6 +317,16 @@ class IPv4Obj(object):
         """Returns the length of the network mask as an integer."""
         return self.prefixlen
 
+    @property
+    def exploded(self):
+        """Returns the IPv4 object in exploded form"""
+        return self.ip_object.exploded
+
+    @property
+    def packed(self):
+        """Returns the IPv4 object in packed binary form"""
+        return self.ip_object.packed
+
     @property
     def broadcast(self):
         """Returns the broadcast address as an IPv4Address object."""
@@ -344,6 +397,16 @@ class IPv4Obj(object):
         return tuple(['{0:02x}'.format(int(num)) for num in \
             str(self.ip).split('.')])
 
+    @property
+    def as_cidr_addr(self):
+        """Returns a string with the address in CIDR notation"""
+        return str(self.ip)+'/'+str(self.prefixlen)
+
+    @property
+    def as_cidr_net(self):
+        """Returns a string with the network in CIDR notation"""
+        return str(self.network)
+
     @property
     def is_multicast(self):
         """Returns a boolean for whether this is a multicast address"""
@@ -410,6 +473,11 @@ class IPv6Obj(object):
                 self.network_object = IPv6Network(str(arg)+'/128')
                 self.ip_object = IPv6Address(str(arg).split('/')[0])
                 return None
+            elif isinstance(arg, int):
+                self.ip_object = IPv6Address(arg)
+                self.network_object = IPv6Network(str(self.ip_object)+'/128',
+                    strict=False)
+                return None
             else:
                 raise ValueError(
                     "IPv6Obj doesn't understand how to parse {0}".format(arg))
@@ -436,19 +504,19 @@ class IPv6Obj(object):
 
     def __gt__(self, val):
         try:
-            val_prefixlen = int(getattr(val, 'prefixlen'))
+            val_in_self = self.__contains__(val)
             val_nobj = getattr(val, 'network_object')
+            self_nobj = getattr(self, 'network_object')
+            val_dec = getattr(val, 'as_decimal')
+            self_dec = getattr(self, 'as_decimal')
 
-            self_nobj = self.network_object
-            if (self.network_object.prefixlen < val_prefixlen):
+            if val_in_self:
                 # Sort shorter masks as higher...
-                return True
-            elif (self.network_object.prefixlen > val_prefixlen):
                 return False
-            elif (self_nobj > val_nobj):
-                # If masks are equal, rely on Google's sorting...
-                return True
-            return False
+            elif self_nobj==val_nobj:
+                return (self_dec > val_dec)
+            else:
+                return (self_nobj > val_nobj)
         except:
             errmsg = "{0} cannot compare itself to '{1}'".format(
                 self.__repr__(), val)
@@ -456,24 +524,34 @@ class IPv6Obj(object):
 
     def __lt__(self, val):
         try:
-            val_prefixlen = int(getattr(val, 'prefixlen'))
+            val_in_self = self.__contains__(val)
             val_nobj = getattr(val, 'network_object')
+            self_nobj = getattr(self, 'network_object')
+            val_dec = getattr(val, 'as_decimal')
+            self_dec = getattr(self, 'as_decimal')
 
-            self_nobj = self.network_object
-            if (self.network_object.prefixlen > val_prefixlen):
-                # Sort shorter masks as lower...
+            if val_in_self:
+                # Sort shorter masks as higher...
                 return True
-            elif (self.network_object.prefixlen < val_prefixlen):
-                return False
-            elif (self_nobj < val_nobj):
-                # If masks are equal, rely on Google's sorting...
-                return True
-            return False
+            elif self_nobj==val_nobj:
+                return (self_dec < val_dec)
+            else:
+                return (self_nobj < val_nobj)
         except:
             errmsg = "{0} cannot compare itself to '{1}'".format(
                 self.__repr__(), val)
             raise ValueError(errmsg)
 
+    def __add__(self, val):
+        """Add an integer to IPv6Obj() and return an integer"""
+        assert isinstance(val, int)
+        return self.as_decimal + val
+
+    def __sub__(self, val):
+        """Subtract an integer from IPv6Obj() and return an integer"""
+        assert isinstance(val, int)
+        return self.as_decimal - val
+
     def __contains__(self, val):
         # Used for "foo in bar"... python calls bar.__contains__(foo)
         try:
@@ -484,9 +562,10 @@ class IPv6Obj(object):
                 #    val, this object cannot contain val
                 return False
             else:
+                # NOTE: We cannot use the same algorithm as IPv4Obj.__contains__() because IPv6Obj doesn't have .broadcast
                 #return (val.network in self.network)
                 return (self.network<=val.network) and \
-                    (self.broadcast>=val.broadcast)
+                    ((self.as_decimal+self.numhosts-1)>=(val.as_decimal+val.numhosts-1))
 
         except (Exception) as e:
             raise ValueError(
@@ -536,12 +615,12 @@ class IPv6Obj(object):
     @property
     def exploded(self):
         """Returns the IPv6 object in exploded form"""
-        return self.network_object.exploded
+        return self.ip_object.exploded
 
     @property
     def packed(self):
-        """Returns the IPv6 object in packed form"""
-        return self.network_object.packed
+        """Returns the IPv6 object in packed binary form"""
+        return self.ip_object.packed
 
     @property
     def broadcast(self):
@@ -601,6 +680,16 @@ class IPv6Obj(object):
         ]
         return tuple(itertools.chain(*nested_list))
 
+    @property
+    def as_cidr_addr(self):
+        """Returns a string with the address in CIDR notation"""
+        return str(self.ip)+'/'+str(self.prefixlen)
+
+    @property
+    def as_cidr_net(self):
+        """Returns a string with the network in CIDR notation"""
+        return str(self.network)
+
     @property
     def is_multicast(self):
         """Returns a boolean for whether this is a multicast address"""
@@ -699,26 +788,73 @@ class L4Object(object):
     def __repr__(self):
         return "<L4Object {0} {1}>".format(self.protocol, self.port_list)
 
-def dns_query(input="", record_type="", server="", timeout=2.0):
-    """A unified IPv4 & IPv6 DNS lookup interface
+class DNSResponse(object):
+    """A universal DNS Response object
+
+    Kwargs:
+        - query_type (str): A string containing the DNS record to lookup
+        - result_str (str): A string containing the DNS Response
+        - input (str): The DNS query string
+        - duration (float): The query duration in seconds 
+
+    Attributes:
+        - query_type (str): A string containing the DNS record to lookup
+        - result_str (str): A string containing the DNS Response
+        - input (str): The DNS query string
+        - has_error (bool): Indicates the query resulted in an error when True
+        - error_str (str): The error returned by dnspython
+        - duration (float): The query duration in seconds 
+        - preference (int): The MX record's preference (default: -1)
+
+    Returns:
+        A :class:`~ccp_util.DNSResponse` instance
+"""
+    def __init__(self, query_type="", result_str="", input="", duration=0.0):
+        self.query_type = query_type
+        self.result_str = result_str
+        self.input = input
+        self.duration = duration  # Query duration in seconds
+
+        self.has_error = False
+        self.error_str = ""
+        self.preference = -1   # MX Preference
+
+    def __str__(self):
+        return self.result_str
+
+    def __repr__(self):
+        if not self.has_error:
+            return '<DNSResponse "{0}" result_str="{1}">'.format(
+                self.query_type, self.result_str)
+        else:
+            return '<DNSResponse "{0}" error="{1}">'.format(
+                self.query_type, self.error_str)
+
+def dns_query(input="", query_type="", server="", timeout=2.0):
+    """A unified IPv4 & IPv6 DNS lookup interface; this is essentially just a wrapper around dnspython's API.  When you query a PTR record, you can use an IPv4 or IPv6 address (which will automatically be converted into an in-addr.arpa name.  This wrapper only supports a subset of DNS records: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', and 'TXT'
 
     Kwargs:
         - input (str): A string containing the DNS record to lookup
-        - record_type (str): A string containing the DNS record type (SOA not supported)
+        - query_type (str): A string containing the DNS record type (SOA not supported)
         - server (str): A string containing the fqdn or IP address of the dns server
-        - timeout (float): DNS lookup timeout duration
+        - timeout (float): DNS lookup timeout duration (default: 2.0 seconds)
 
     Returns:
-        - a dictionary with these keys:
-          'value': A python set() of the query answers
-          'record': The record type
-          'error': A string containing any errors reported (default: '')
-          'input': A string containing the input request
+        A set() of :class:`~ccp_util.DNSResponse` instances
+
+>>> from ciscoconfparse.ccp_util import dns_query
+>>> dns_query('www.pennington.net', "A", "4.2.2.2")
+set([<DNSResponse "A" result_str="65.19.187.2">])
+>>> answer = dns_query('www.pennington.net', 'A', '4.2.2.2')
+>>> str(answer.pop())
+'65.19.187.2'
+>>>
     """
-        
-    valid_records = set(['A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'TXT'])
-    record_type = record_type.upper()
-    assert record_type in valid_records
+
+    valid_records = set(['A', 'AAAA', 'AXFR', 'CNAME', 'MX', 'NS', 'PTR', 
+        'TXT'])
+    query_type = query_type.upper()
+    assert query_type in valid_records
     assert server!=""
     assert float(timeout)>0
     assert input != ""
@@ -728,51 +864,76 @@ def dns_query(input="", record_type="", server="", timeout=2.0):
     resolver.server = [socket.gethostbyname(server)]
     resolver.timeout = float(timeout)
     resolver.lifetime = float(timeout)
-    if (record_type=="A") or (record_type=="AAAA"):
+    start = time.time()
+    if (query_type=="A") or (query_type=="AAAA"):
         try:
-            for result in resolver.query(input, record_type):
-                retval.add(str(result.address))
+            answer = resolver.query(input, query_type)
+            duration = time.time() - start
+            for result in answer:
+                response = DNSResponse(query_type=query_type, 
+                    duration=duration,
+                    input=input, result_str = str(result.address))
+                retval.add(response)
         except DNSException as e:
-            return {
-                'value': '',
-                'record': record_type,
-                'error': e,
-                'input': input,
-                }
-    elif record_type=="CNAME":
+                duration = time.time() - start
+                response = DNSResponse(input=input, 
+                    duration=duration, query_type=query_type)
+                response.has_error = True
+                response.error_str = e
+                retval.add(response)
+    elif query_type=="AXFR":
+        """This is a hack: return text of zone transfer, instead of axfr objs"""
+        _zone = zone.from_xfr(query.xfr(server, input, lifetime=timeout))
+        return [_zone[node].to_text(node) for node in _zone.nodes.keys()]
+    elif query_type=="CNAME":
         try:
-            for result in resolver.query(input, record_type):
-                retval.add(str(result.target))
+            answer = resolver.query(input, query_type)
+            duration = time.time() - start
+            for result in answer:
+                response = DNSResponse(query_type=query_type, 
+                    duration=duration, 
+                    input=input, result_str = str(result.target))
+                retval.add(response)
         except DNSException as e:
-            return {
-                'value': '',
-                'record': record_type,
-                'error': e,
-                'input': input,
-                }
-    elif record_type=="MX":
+                duration = time.time() - start
+                response = DNSResponse(input=input, duration=duration,
+                    query_type=query_type)
+                response.has_error = True
+                response.error_str = e
+                retval.add(response)
+    elif query_type=="MX":
         try:
-            for result in resolver.query(input, record_type):
-                retval.add(str(result.exchange))
+            answer = resolver.query(input, query_type)
+            duration = time.time() - start
+            for result in answer:
+                response = DNSResponse(query_type=query_type, 
+                    input=input, result_str = str(result.target))
+                response.preference = int(result.preference)
+                retval.add(response)
         except DNSException as e:
-            return {
-                'value': '',
-                'record': record_type,
-                'error': e,
-                'input': input,
-                }
-    elif record_type=="NS":
+                duration = time.time() - start
+                response = DNSResponse(input=input, duration=duration,
+                    query_type=query_type)
+                response.has_error = True
+                response.error_str = e
+                retval.add(response)
+    elif query_type=="NS":
         try:
-            for result in resolver.query(input, record_type):
-                retval.add(str(result.target))
+            answer = resolver.query(input, query_type)
+            duration = time.time() - start
+            for result in answer:
+                response = DNSResponse(query_type=query_type, 
+                    duration=duration,
+                    input=input, result_str = str(result.target))
+                retval.add(response)
         except DNSException as e:
-            return {
-                'value': '',
-                'record': record_type,
-                'error': e,
-                'input': input,
-                }
-    elif record_type=="PTR":
+                duration = time.time() - start
+                response = DNSResponse(input=input, 
+                    duration=duration, query_type=query_type)
+                response.has_error = True
+                response.error_str = e
+                retval.add(response)
+    elif query_type=="PTR":
         if is_valid_ipv4_addr(input) or is_valid_ipv6_addr(input):
             inaddr = reversename.from_address(input)
         elif 'in-addr.arpa' in input.lower():
@@ -781,32 +942,38 @@ def dns_query(input="", record_type="", server="", timeout=2.0):
             raise ValueError('Cannot query PTR record for "{0}"'.format(input))
 
         try:
-            for result in resolver.query(inaddr, record_type):
-                retval.add(str(result.target))
+            answer = resolver.query(inaddr, query_type)
+            duration = time.time() - start
+            for result in answer:
+                response = DNSResponse(query_type=query_type, 
+                    duration=duration,
+                    input=inaddr, result_str = str(result.target))
+                retval.add(response)
         except DNSException as e:
-            return {
-                'value': '',
-                'record': record_type,
-                'error': e,
-                'input': input,
-                }
-    elif record_type=="TXT":
+                duration = time.time() - start
+                response = DNSResponse(input=input, 
+                    duration=duration,
+                    query_type=query_type)
+                response.has_error = True
+                response.error_str = e
+                retval.add(response)
+    elif query_type=="TXT":
         try:
-            for result in resolver.query(input, record_type):
-                retval.add(str(result.strings))
+            answer = resolver.query(input, query_type)
+            duration = time.time() - start
+            for result in answer:
+                response = DNSResponse(query_type=query_type, 
+                    duration=duration,
+                    input=inaddr, result_str = str(result.strings))
+                retval.add(response)
         except DNSException as e:
-            return {
-                'value': '',
-                'record': record_type,
-                'error': e,
-                'input': input,
-                }
-    return {
-        'value': retval,
-        'record': record_type,
-        'error': '',
-        'input': input,
-        }
+                duration = time.time() - start
+                response = DNSResponse(input=input, 
+                    duration=duration, query_type=query_type)
+                response.has_error = True
+                response.error_str = e
+                retval.add(response)
+    return retval
 
 def dns_lookup(input, timeout=3, server=''):
     """Perform a simple DNS lookup, return results in a dictionary"""
@@ -854,7 +1021,6 @@ def dns6_lookup(input, timeout=3, server=''):
 
 _REVERSE_DNS_REGEX = re.compile(r'^\s*\d+\.\d+\.\d+\.\d+\s*$')
 
-
 def reverse_dns_lookup(input, timeout=3, server=''):
     """Perform a simple reverse DNS lookup, return results in a dictionary"""
     assert _REVERSE_DNS_REGEX.search(
@@ -889,23 +1055,29 @@ class CiscoRange(MutableSequence):
 
 >>> from ciscoconfparse.ccp_util import CiscoRange
 >>> CiscoRange('1-3,5,9-11,13')
-<CiscoRange ['1', '2', '3', '5', '9', '10', '11', '13']>
+<CiscoRange 1-3,5,9-11,13>
 >>> CiscoRange('Eth1/1-3,7')
-<CiscoRange ['Eth1/1', 'Eth1/2', 'Eth1/3', 'Eth1/7']>
+<CiscoRange Eth1/1-3,7>
+>>> CiscoRange()
+<CiscoRange none>
     """
-    def __init__(self, text, result_type=str):
+    def __init__(self, text="", result_type=str):
         super(CiscoRange, self).__init__()
         self.text = text
         self.result_type = result_type
         if text:
-            self.line_prefix, self.slot_prefix, self.range_text = self._parse_range_text(
-            )
+            self.line_prefix, self.slot_prefix, self.range_text = self._parse_range_text()
             self._list = self._range()
         else:
+            self.line_prefix = ""
+            self.slot_prefix = ""
             self._list = list()
 
     def __repr__(self):
-        return """<CiscoRange {0}>""".format(self._list)
+        if len(self._list)==0:
+            return """<CiscoRange none>"""
+        else:
+            return """<CiscoRange {0}>""".format(self.compressed_str)
 
     def __len__(self):
         return len(self._list)
@@ -922,12 +1094,24 @@ class CiscoRange(MutableSequence):
     def __str__(self):
         return self.__repr__()
 
+    # Github issue #124
+    def __eq__(self, other):
+        assert hasattr(other, 'line_prefix')
+        self_prefix_str = self.line_prefix + self.slot_prefix
+        other_prefix_str = other.line_prefix + other.slot_prefix
+        cmp1 = self_prefix_str.lower()==other_prefix_str.lower()
+        cmp2 = sorted(self._list)==sorted(other._list)
+        return cmp1 and cmp2
+
     def insert(self, ii, val):
         ## Insert something at index ii
         for idx, obj in enumerate(
                 CiscoRange(
                     val, result_type=self.result_type)):
             self._list.insert(ii + idx, obj)
+
+        # Prune out any duplicate entries, and sort...
+        self._list = sorted(map(self.result_type, set(self._list)))
         return self
 
     def append(self, val):
@@ -953,7 +1137,8 @@ class CiscoRange(MutableSequence):
             range_text = mm_result['range_text']
         return line_prefix, slot_prefix, range_text
 
-    def _dash_range(self, text):
+    def _parse_dash_range(self, text):
+        """Parse a dash Cisco range into a discrete list of items"""
         retval = list()
         for range_atom in text.split(','):
             try:
@@ -973,14 +1158,16 @@ class CiscoRange(MutableSequence):
         def combine(arg):
             return self.line_prefix + self.slot_prefix + str(arg)
 
-        return [self.result_type(ii) for ii in map(combine, self._dash_range(self.range_text))]
+        return [self.result_type(ii) for ii in map(combine, self._parse_dash_range(self.range_text))]
 
     def remove(self, arg):
         remove_obj = CiscoRange(arg)
         for ii in remove_obj:
             try:
-                index = self.index(self.result_type(ii))
-                self.pop(index)
+                ## Remove arg, even if duplicated... Ref Github issue #126
+                while True:
+                    index = self.index(self.result_type(ii))
+                    self.pop(index)
             except ValueError:
                 pass
         return self
@@ -988,3 +1175,55 @@ class CiscoRange(MutableSequence):
     @property
     def as_list(self):
         return self._list
+
+    ## Github issue #125
+    @property
+    def compressed_str(self):
+        """Return a text string with a compressed csv of values
+
+>>> from ciscoconfparse.ccp_util import CiscoRange
+>>> range_obj = CiscoRange('1,3,5,6,7')
+>>> range_obj.compressed_str
+'1,3,5-7'
+>>>
+        """
+        retval = list()
+        prefix_str = self.line_prefix + self.slot_prefix
+
+        # Build a list of integers (without prefix_str)
+        input = list()
+        for ii in self._list:
+            try:
+                unicode_ii = str(ii, 'utf-8') # Python2.7...
+            except:
+                unicode_ii = str(ii)
+            ii = re.sub(r'^{0}(\d+)$'.format(prefix_str), '\g<1>', unicode_ii)
+            input.append(int(ii))
+
+        if len(input)==0: # Special case, handle empty list
+            return ''
+
+        # source - https://stackoverflow.com/a/51227915/667301
+        input = sorted(list(set(input)))
+        range_list = [input[0]]
+        for ii in range(len(input)):
+            if ii+1<len(input) and ii-1>-1:
+                if (input[ii]-input[ii-1]==1) and (input[ii+1]-input[ii]==1):
+                    if  range_list[-1]!='-':
+                        range_list += ['-']
+                    else:
+                        range_list = range_list
+                else:
+                        range_list+=[input[ii]]
+        if len(input)>1:
+            range_list+=[input[len(input)-1]]
+
+        # Build the return value from range_list...
+        retval = prefix_str + str(range_list[0])
+        for ii in range(1,len(range_list)):
+            if str(type(range_list[ii]))!=str(type(range_list[ii-1])):
+                retval += str(range_list[ii])
+            else:
+                retval += ',' + str(range_list[ii])
+
+        return retval
