@@ -1,14 +1,15 @@
+from __future__  import absolute_import
 import traceback
 import sys
 import re
 import os
 
-from errors import DynamicAddressException
+from ciscoconfparse.errors import DynamicAddressException
 
-from ccp_util import _IPV6_REGEX_STR_COMPRESSED1, _IPV6_REGEX_STR_COMPRESSED2
-from ccp_util import _IPV6_REGEX_STR_COMPRESSED3
-from ccp_util import CiscoRange, IPv4Obj, IPv6Obj
-from ccp_abc import BaseCfgLine
+from ciscoconfparse.ccp_util import _IPV6_REGEX_STR_COMPRESSED1, _IPV6_REGEX_STR_COMPRESSED2
+from ciscoconfparse.ccp_util import _IPV6_REGEX_STR_COMPRESSED3
+from ciscoconfparse.ccp_util import CiscoRange, IPv4Obj, IPv6Obj
+from ciscoconfparse.ccp_abc import BaseCfgLine
 
 ### HUGE UGLY WARNING:
 ###   Anything in models_cisco.py could change at any time, until I remove this
@@ -18,7 +19,7 @@ from ccp_abc import BaseCfgLine
 ###
 ###   Use models_cisco.py at your own risk.  You have been warned :-)
 """ models_cisco.py - Parse, Query, Build, and Modify IOS-style configurations
-     Copyright (C) 2014-2018 David Michael Pennington
+     Copyright (C) 2014-2019 David Michael Pennington
 
      This program is free software: you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
@@ -37,6 +38,8 @@ from ccp_abc import BaseCfgLine
      mike [~at~] pennington [/dot\] net
 """
 
+MAX_VLAN = 4094
+
 ##
 ##-------------  IOS Configuration line object
 ##
@@ -158,10 +161,10 @@ class IOSCfgLine(BaseCfgLine):
            ...     '!',
            ...     ]
            >>> parse = CiscoConfParse(config)
-           >>> obj = parse.find_objects('^interface\sSerial')[0]
+           >>> obj = parse.find_objects(r'^interface\sSerial')[0]
            >>> obj.is_subintf
            False
-           >>> obj = parse.find_objects('^interface\sATM')[0]
+           >>> obj = parse.find_objects(r'^interface\sATM')[0]
            >>> obj.is_subintf
            True
            >>>
@@ -203,10 +206,10 @@ class IOSCfgLine(BaseCfgLine):
            ...     '!',
            ...     ]
            >>> parse = CiscoConfParse(config)
-           >>> obj = parse.find_objects('^interface\sFast')[0]
+           >>> obj = parse.find_objects(r'^interface\sFast')[0]
            >>> obj.is_loopback_intf
            False
-           >>> obj = parse.find_objects('^interface\sLoop')[0]
+           >>> obj = parse.find_objects(r'^interface\sLoop')[0]
            >>> obj.is_loopback_intf
            True
            >>>
@@ -260,7 +263,7 @@ class IOSCfgLine(BaseCfgLine):
         return False
 
     @property
-    def in_portchannel(self):
+    def intf_in_portchannel(self):
         """Return a boolean indicating whether this port is configured in a port-channel
 
         """
@@ -278,7 +281,7 @@ class IOSCfgLine(BaseCfgLine):
         return retval
 
     @property
-    def is_portchannel(self):
+    def is_portchannel_intf(self):
         """Return a boolean indicating whether this port is a port-channel intf
 
         """
@@ -739,10 +742,14 @@ class BaseIOSIntfLine(IOSCfgLine):
             r'^\s*carrier-delay\s+msec\s+(\d+)$',
             result_type=float,
             default=0.0)
+
         if (cd_seconds > 0.0):
             return cd_seconds
-        else:
+        elif (cd_msec > 0.0):
             return cd_msec / 1000.0
+        else:
+            return 0.0
+
 
     @property
     def has_manual_clock_rate(self):
@@ -1049,7 +1056,7 @@ class BaseIOSIntfLine(IOSCfgLine):
             return False
 
         retval = self.re_match_iter_typed(
-            r'^\s*ip\spim\sdense-mode\s*$)\s*$',
+            r'^\s*(ip\spim\sdense-mode)\s*$',
             result_type=bool,
             default=False)
         return retval
@@ -1065,7 +1072,7 @@ class BaseIOSIntfLine(IOSCfgLine):
             return False
 
         retval = self.re_match_iter_typed(
-            r'^\s*ip\spim\ssparse-mode\s*$)\s*$',
+            r'^\s*(ip\spim\ssparse-mode)\s*$',
             result_type=bool,
             default=False)
         return retval
@@ -1081,7 +1088,7 @@ class BaseIOSIntfLine(IOSCfgLine):
             return False
 
         retval = self.re_match_iter_typed(
-            r'^\s*ip\spim\ssparse-dense-mode\s*$)\s*$',
+            r'^\s*(ip\spim\ssparse-dense-mode)\s*$',
             result_type=bool,
             default=False)
         return retval
@@ -1103,6 +1110,43 @@ class BaseIOSIntfLine(IOSCfgLine):
             r'^\s*arp\s+timeout\s+(\d+)\s*$', result_type=int, default=0)
         return retval
 
+    @property
+    def has_ip_helper_addresses(self):
+        """Return a True if the intf has helper-addresses; False if not"""
+        if len(self.ip_helper_addresses)>0:
+            return True
+        return False
+
+    @property
+    def ip_helper_addresses(self):
+        """Return a list of dicts with IP helper-addresses.  Each helper-address is in a dictionary.  The dictionary is in this format:
+
+        .. code-block:: python
+           :emphasize-lines: 11
+
+           >>> config = [
+           ...     '!',
+           ...     'interface FastEthernet1/1',
+           ...     ' ip address 1.1.1.1 255.255.255.0',
+           ...     ' ip helper-address 172.16.20.12',
+           ...     ' ip helper-address 172.19.185.91',
+           ...     '!',
+           ...     ]
+           >>> parse = CiscoConfParse(config)
+           >>> obj = parse.find_objects('^interface\sFastEthernet1/1$')[0]
+           >>> obj.ip_helper_addresses
+           [{'addr': '172.16.20.12', 'vrf': '', 'global': False}, {'addr': '172.19.185.91', 'vrf': '', 'global': False}]
+           >>>"""
+        retval = list()
+        for child in self.children:
+            if 'helper-address' in child.text:
+                addr = child.re_match_typed(r'ip\s+helper-address\s.*?(\d+\.\d+\.\d+\.\d+)')
+                global_addr = child.re_match_typed(r'ip\s+helper-address\s+(global)', result_type=bool, default=False)
+                vrf = child.re_match_typed(r'ip\s+helper-address\s+vrf\s+(\S+)',
+                    default='')
+                retval.append({'addr': addr, 'vrf': vrf, 'global': bool(global_addr)})
+        return retval
+
     @property
     def is_switchport(self):
         retval = self.re_match_iter_typed(
@@ -1187,54 +1231,63 @@ class BaseIOSIntfLine(IOSCfgLine):
 
     @property
     def trunk_vlans_allowed(self):
-        """Return a CiscoRange() with the list of allowed vlan numbers.  Return 0 if the port isn't a switchport"""
-        if self.is_switchport:
-            default_val = '1-4094'
+        """Return a CiscoRange() with the list of allowed vlan numbers (as int).  Return 0 if the port isn't a switchport in trunk mode"""
+
+        # The default values...
+        if self.is_switchport and not self.has_manual_switch_access:
+            retval = CiscoRange('1-{0}'.format(MAX_VLAN), result_type=int)
         else:
-            default_val = 0
+            return 0
 
-        allowed = self.re_match_iter_typed(
-            r'^\s*switchport\s+trunk\s+allowed\s+vlan\s+(\S+)$',
-            result_type=str,
-            default='1-4094')
-        if allowed == 'all':
-            allowed = '1-4094'
-        elif allowed == 'none':
-            allowed = ''
-        add = self.re_match_iter_typed(
-            r'^\s*switchport\s+trunk\s+allowed\s+vlan\s+add\s+(\S+)$',
-            result_type=str,
-            default='')
+        ## Iterate over switchport trunk statements
+        for obj in self.children:
 
-        if allowed and add:
-            combined = allowed + ',' + add
-        elif allowed:
-            combined = allowed
-        elif add:
-            combined = add
-        else:
-            combined = allowed
+            ## For every child object, check whether the vlan list is modified
+            abs_str = obj.re_match_typed(
+                '^\s+switchport\s+trunk\s+allowed\s+vlan\s(all|none|\d.*?)$',
+                default='_nomatch_', result_type=str).lower()
+            add_str = obj.re_match_typed(
+                '^\s+switchport\s+trunk\s+allowed\s+vlan\s+add\s+(\d.*?)$',
+                default='_nomatch_', result_type=str).lower()
+            exc_str = obj.re_match_typed(
+                '^\s+switchport\s+trunk\s+allowed\s+vlan\s+except\s+(\d.*?)$',
+                default='_nomatch_', result_type=str).lower()
+            rem_str = obj.re_match_typed(
+                '^\s+switchport\s+trunk\s+allowed\s+vlan\s+remove\s+(\d.*?)$',
+                default='_nomatch_', result_type=str).lower()
 
-        remove = self.re_match_iter_typed(
-            r'^\s*switchport\s+trunk\s+allowed\s+vlan\s+remove\s+(\S+)$',
-            result_type=str,
-            default='')
+            ## Build a vdict for each vlan modification statement
+            vdict = {
+                'absolute_str': abs_str,
+                'add_str': add_str,
+                'except_str': exc_str,
+                'remove_str': rem_str,
+            }
 
-        if remove:
-            retval = CiscoRange(combined, result_type=int).remove(remove)
-        else:
-            retval = CiscoRange(combined, result_type=int)
-
-        _except = self.re_match_iter_typed(
-            r'^\s*switchport\s+trunk\s+allowed\s+vlan\s+except\s+(\S+)$',
-            result_type=str,
-            default='')
-
-        if _except:
-            retval = CiscoRange(combined, result_type=int).remove(_except)
+            ## Analyze each vdict in sequence and apply to retval sequentially
+            for key, val in vdict.items():
+                if val!='_nomatch_':
+                    ## absolute in the key overrides previous values
+                    if 'absolute' in key:
+                        if val.lower()=='all':
+                            retval = CiscoRange('1-{0}'.format(MAX_VLAN), 
+                                result_type=int)
+                        elif val.lower()=='none':
+                            retval = CiscoRange(result_type=int)
+                        else:
+                            retval = CiscoRange(val, result_type=int)
+                    elif 'add' in key:
+                        retval.append(val)
+                    elif 'except' in key:
+                        retval = CiscoRange('1-{0}'.format(MAX_VLAN), 
+                            result_type=int)
+                        retval.remove(val)
+                    elif 'remove' in key:
+                        retval.remove(val)
 
         return retval
 
+
     @property
     def native_vlan(self):
         """Return an integer with the native vlan number.  Return 1, if the switchport has no explicit native vlan configured; return 0 if the port isn't a switchport"""
@@ -1334,22 +1387,26 @@ class BaseIOSIntfLine(IOSCfgLine):
     def hsrp_hello_timer(self):
         ## For API simplicity, I always assume there is only one hsrp 
         ##     group on the interface
+
+        # FIXME: handle msec timers...
         retval = self.re_match_iter_typed(
             r'^\s*standby\s+(\d+\s+)*timers\s+(\d+)\s+\d+',
             group=2,
-            result_type=int,
-            default=0)
+            result_type=float,
+            default=0.0)
         return retval
 
     @property
     def hsrp_hold_timer(self):
         ## For API simplicity, I always assume there is only one hsrp 
         ##     group on the interface
+
+        # FIXME: this should be a float (in case of msec timers)
         retval = self.re_match_iter_typed(
             r'^\s*standby\s+(\d+\s+)*timers\s+\d+\s+(\d+)',
             group=2,
-            result_type=int,
-            default=0)
+            result_type=float,
+            default=0.0)
         return retval
 
     @property
@@ -1497,6 +1554,7 @@ class IOSIntfLine(BaseIOSIntfLine):
           All :class:`~models_cisco.IOSIntfLine` methods are still considered beta-quality, until this notice is removed.  The behavior of APIs on this object could change at any time.
         """
         super(IOSIntfLine, self).__init__(*args, **kwargs)
+        self.feature = 'interface'
 
     @classmethod
     def is_object_for(cls, line="", re=re):
@@ -1754,6 +1812,8 @@ _RE_IPV6_ROUTE = re.compile(r"""^ipv6\s+route
 (?:\s+(?P<ad>\d+))?              # Administrative distance
 (?:\s+(?:(?P<ucast>unicast)|(?P<mcast>multicast)))?
 (?:\s+tag\s+(?P<tag>\d+))?       # Route tag
+(?:\s+track\s+(?P<track>\d+))?   # Track object
+(?:\s+name\s+(?P<name>\S+))?     # Route name
 """.format(_IPV6_REGEX_STR_COMPRESSED1, _IPV6_REGEX_STR_COMPRESSED2,
            _IPV6_REGEX_STR_COMPRESSED3), re.VERBOSE)
 
@@ -1915,13 +1975,10 @@ class IOSRouteLine(BaseIOSRouteLine):
 
     @property
     def route_name(self):
-        if self._address_family == 'ip':
-            if self.route_info['name']:
-                return self.route_info['name']
-            else:
-                return ''
-        elif self._address_family == 'ipv6':
-            raise NotImplementedError
+        if self.route_info['name']:
+            return self.route_info['name']
+        else:
+            return ''
 
     @property
     def permanent(self):
@@ -1935,13 +1992,10 @@ class IOSRouteLine(BaseIOSRouteLine):
 
     @property
     def tracking_object_name(self):
-        if self._address_family == 'ip':
-            if bool(self.route_info['track']):
-                return self.route_info['track']
-            else:
-                return ''
-        elif self._address_family == 'ipv6':
-            raise NotImplementedError
+        if bool(self.route_info['track']):
+            return self.route_info['track']
+        else:
+            return ''
 
     @property
     def tag(self):
